########################################################################################################################
#    File: main.py
#  Author: Dan Huckson, https://github.com/unodan
#    Date: 2018-09-19
# Purpose: MVC (Model View Controller) Demo
########################################################################################################################
from tkinter import Toplevel, Label, Entry, Button


class Teller(Toplevel):  # View 2
    def __init__(self, master, title):
        Toplevel.__init__(self, master)

        self.title(title)

        Label(self, text='Amount').pack(side='left')
        self.amount = Entry(self, width=8)
        self.amount.pack(side='left')

        self.btn_deposit = Button(self, text='Deposit', width=8)
        self.btn_deposit.pack(side='left')

        self.btn_withdrawal = Button(self, text='Withdrawal', width=8)
        self.btn_withdrawal.pack(side='left')



# -*- coding: utf-8 -*-

"""Top-level package for TkMVC."""

from tkmvc.tkmvc import App
from tkmvc.views.bank import Bank
from tkmvc.controllers.controller import Controller

__author__ = """Dan Huckson"""
__email__ = 'danhuckson@hotmail.com'
__version__ = '0.1.15'



# -*- coding: utf-8 -*-
########################################################################################################################
#    File: main.py
#  Author: Dan Huckson, https://github.com/unodan
#    Date: 2018-09-19
########################################################################################################################


from .models import Account
from .views import Bank, Teller
from .controllers import Controller


class App(Controller):  # The Controller
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.account = Account()
        self.account.transaction.add_callback(self.update_account)

        self.views = {'bank': Bank(self, 'The Bank'), 'teller': Teller(self, 'The Teller')}

        self.views['teller'].btn_deposit.config(command=self.make_deposit)
        self.views['teller'].btn_withdrawal.config(command=self.make_withdrawal)

        self.update_account(self.account.transaction.get())

    def view(self, key):
        if key in self.views:
            return self.views[key]

        return False

    def make_deposit(self):
        self.account.deposit(int(self.views['teller'].amount.get()))

    def make_withdrawal(self):
        self.account.withdrawal(int(self.views['teller'].amount.get()))

    def update_account(self, amount):
        self.views['bank'].set_balance(amount)



########################################################################################################################
#    File: bank.py
#  Author: Dan Huckson, https://github.com/unodan
#    Date: 2018-09-20
########################################################################################################################

from tkinter import Toplevel, Label, Entry


class Bank(Toplevel):  # View
    def __init__(self, master, title):
        Toplevel.__init__(self, master)
        self.title(title)

        self.protocol('WM_DELETE_WINDOW', self.master.destroy)

        Label(self, text='Account Balance').pack(side='left')
        self.balance = Entry(self, width=8)
        self.balance.pack(side='left')

    def set_balance(self, amount):
        self.balance.delete(0, 'end')
        self.balance.insert('end', str(amount))




########################################################################################################################
#    File: models.py
#  Author: Dan Huckson, https://github.com/unodan
#    Date: 2018-09-20
########################################################################################################################


class Transaction:
    def __init__(self):
        self.value = 0
        self.callbacks = {}

    def add_callback(self, func):
        self.callbacks[func] = None

    def _callbacks(self):
        for func in self.callbacks:
            func(self.value)

    def set(self, value):
        self.value = value
        self._callbacks()

    def get(self):
        return self.value


class Account:  # The Model
    def __init__(self):
        self.transaction = Transaction()

    def deposit(self, value):
        self.transaction.set(self.transaction.get() + value)

    def withdrawal(self, value):
        self.transaction.set(self.transaction.get() - value)

